/* 
 * Distributed as part of jtmail 1.0
 * 
 * Copyright (C) 2008 QArks.com
 *
 * Author: Pierre Meyer <support@qarks.com>
 *
 * This library is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 2.1, as 
 * published by the Free Software Foundation.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN
 * NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
 * DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
 * OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE
 * USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this software; see the file LICENSE.  If not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 */
package com.qarks.util.mail;

import java.io.File;
import java.io.UnsupportedEncodingException;
import java.util.Date;
import java.util.Properties;

import javax.activation.DataHandler;
import javax.mail.Address;
import javax.mail.Message;
import javax.mail.MessagingException;
import javax.mail.Part;
import javax.mail.Session;
import javax.mail.Transport;
import javax.mail.internet.InternetAddress;
import javax.mail.internet.MimeBodyPart;
import javax.mail.internet.MimeMessage;
import javax.mail.internet.MimeMultipart;


public class SmtpHelper{


	public static void sendMail(MailAttributeBinder attributeBinder,boolean html,String subject, String from, String replyTo,
			String to,String host,int port,String user,String password, 
			String templatePath, String charsetName, ProgressListener progressListener) throws Exception{

		Properties props = System.getProperties();
		props.put("mail.smtp.host", host);
		props.put("mail.smtp.port", new Integer(port));

		MailAuthenticator auth = null;
		if (user != null) {
			auth = new MailAuthenticator(user, password);
		}
		Session session = Session.getInstance(props, auth);
		//content,html,subject,from, to
		MailTarget fromAddress = MailTarget.getMailTarget(from);
		MailTarget toAddress = MailTarget.getMailTarget(to);
		MailTarget replyAddress = MailTarget.getMailTarget(replyTo);
		String content = MailFileHelper.getFileContent(new File(templatePath));

		int index;
		while((index=content.indexOf("%%"))>-1){
			int nextIndex = content.substring(index+2).indexOf("%%");
			if (nextIndex>-1){
				String attributeName = content.substring(index+2,index+2+nextIndex);
				String attributeValue;

				if(attributeBinder.getAttributeValue(attributeName) == null){
					attributeValue="";
				}
				else{
					attributeValue = attributeBinder.getAttributeValue(attributeName);
				}
				content = content.replaceAll("%%"+attributeName+"%%", attributeValue);
			}
		}

		MessageContent messageContent = new MessageContent(fromAddress, replyAddress, 
				new MailTarget[]{toAddress}, 
				new MailTarget[0],
				new MailTarget[0],
				subject, 
				content, 
				new Attachment[0], 
				new Date());

		messageContent.setSendAsText(!html);    
		index = templatePath.lastIndexOf(File.separator);
		String folderPath = templatePath.substring(0,index);

		Message message = createRelatedMessage(session, messageContent, folderPath, charsetName, progressListener);

		Transport transport = null;
		try{
			transport = session.getTransport("smtp");
			transport.connect(host, port, user, password);
			Transport.send(message);
		}
		finally{
			if (transport!=null){
				try {
					transport.close();
				}
				catch (MessagingException ex1) {
				}
			}
		}
	}

	private static Message createRelatedMessage(Session session, MessageContent message, 
			String templatePath, String charsetName, ProgressListener progressListener){

		MimeMessage msg = null;
		String content = "";
		if (message.isSendAsText()){
			content = message.getContent();
		}
		else{
			MailBuilder.extractInlinePictures(message, templatePath, progressListener);
			content = message.getContent();
		}
		SmtpFileDataSource[] fileDataSources = getFileSources(message, progressListener);
		Attachment attachments[] = message.getAttachments();

		try {
			msg = new MimeMessage(session);
			setReceivers(msg,message);
			MimeMultipart multipart = new MimeMultipart("related");

			////////////////////////////////////////////////////////

			MimeBodyPart textPart = new MimeBodyPart();
			String contentType ="";
			if (message.isSendAsText()){
				contentType = "text/plain; charset="+charsetName;
			}
			else{
				contentType = "text/html; charset="+charsetName;
			}
			textPart.setContent(content, contentType);
			multipart.addBodyPart(textPart);

			MimeBodyPart mbps;
			DataHandler dh;
			if (message.isSendAsText()==false){
				// it is useless to add the inline attachments if we are sending as text, because
				// anyway all <img src=...> will have been removed
				for (int i = 0; i < fileDataSources.length; i++) {
					if (attachments[i].isInline()) {
						dh = new DataHandler(fileDataSources[i]);
						mbps = new MimeBodyPart();
						mbps.setDataHandler(dh);
						mbps.setFileName(attachments[i].getDisplayName());

						String name = attachments[i].getFullPath().toLowerCase();
						if (name.endsWith(".jpg")) {
							mbps.addHeader("Content-Type", "image/jpeg");
						}
						else if (name.endsWith(".gif")) {
							mbps.addHeader("Content-Type", "image/gif");
						}
						else if (name.endsWith(".png")) {
							mbps.addHeader("Content-Type", "image/png");
						}

						mbps.setDisposition(Part.INLINE);
						mbps.addHeader("Content-ID", "<" + attachments[i].getDisplayName() + ">");
						multipart.addBodyPart(mbps);
					}
				}
			}

			//////////////////////////////////////////////////////////

			for (int i = 0; i < fileDataSources.length; i++) {
				if (attachments[i].isInline()==false){
					dh = new DataHandler(fileDataSources[i]);
					mbps = new MimeBodyPart();
					mbps.setDataHandler(dh);
					mbps.setFileName(attachments[i].getDisplayName());
					mbps.setDisposition(Part.ATTACHMENT);
					multipart.addBodyPart(mbps);
				}
			}

			//////////////////////////////////////////////////////////


			msg.setContent(multipart);
			msg.addHeader("X-Mailer","JavaMail");
			msg.setSentDate(new Date());
			msg.saveChanges();
		}
		catch (Throwable ex) {
			ex.printStackTrace();
			msg = null;
		}
		return msg;
	}

	private static SmtpFileDataSource[] getFileSources(MessageContent message, ProgressListener progressListener){
		SmtpFileDataSource[] fileDataSources;
		Attachment mAttachments[] = message.getAttachments();
		File mFiles[] = new File[mAttachments.length];
		for(int i=0;i<mFiles.length;i++){
			mFiles[i] = new File(mAttachments[i].getFullPath());
		}

		if (mFiles != null) {
			fileDataSources = new SmtpFileDataSource[mFiles.length];
			for (int i = 0; i < mFiles.length; i++) {
				fileDataSources[i] = new SmtpFileDataSource(mFiles[i],progressListener);
			}
		}
		else{
			fileDataSources = new SmtpFileDataSource[0];
		}
		return fileDataSources;
	}

	private static void setReceivers(Message msg, MessageContent message) throws MessagingException, UnsupportedEncodingException, MessagingException {
		MailTarget mTos[] = message.getTos();
		MailTarget mCcs[] = message.getCcs();
		MailTarget mBccs[] = message.getBccs();
		String mSubject = message.getSubject();
		MailTarget mFrom = message.getFrom();
		MailTarget mReplyTo = message.getReplyTo();
		msg.setFrom(new InternetAddress(mFrom.getAddress(),mFrom.getDisplayString()));
		msg.setReplyTo(new Address[]{new InternetAddress(mReplyTo.getAddress(),mReplyTo.getDisplayString())});

		InternetAddress tos[] = new InternetAddress[mTos.length];
		for (int i = 0; i < tos.length; i++) {
			tos[i] = new InternetAddress(mTos[i].getAddress(),mTos[i].getDisplayString());
		}
		msg.setRecipients(Message.RecipientType.TO, tos);
		InternetAddress ccs[] = new InternetAddress[mCcs.length];
		for (int i = 0; i < ccs.length; i++) {
			ccs[i] = new InternetAddress(mCcs[i].getAddress(),mCcs[i].getDisplayString());
		}
		msg.setRecipients(Message.RecipientType.CC, ccs);
		InternetAddress bccs[] = new InternetAddress[mBccs.length];
		for (int i = 0; i < bccs.length; i++) {
			bccs[i] = new InternetAddress(mBccs[i].getAddress(),mBccs[i].getDisplayString());
		}
		msg.setRecipients(Message.RecipientType.BCC, bccs);
		msg.setSubject(mSubject);
	}
}
